B0	equ	20H			; Reception buffer 
B1	equ	21H
B2	equ	22H
B3	equ	23H

	org	0000H
OnRst:	ljmp	Init			; On Reset 


	org	000bH			
OnT0:	ljmp	T0Han			; On Timer 0 interrupt 


	org	0023H
OnSI:	ljmp	SIHan			; On Serial Interface interrupt 	


;============================================================================
; Obsluha preruseni od casovace T0 
;----------------------------------------------------------------------------

T0Han:  mov	TH0,	#246		; T0 MSB reload  (400 Hz)
	mov	TL0,	#255		; T0 LSB reload
	push	PSW
	push	Acc
	mov	B,	#80H
	mov	P0, 	#10111111B	; Send bits  0 06 05 04 03 02 01 00
	nop
	mov	A,  	P2
	mov	P0, 	#11111111B
	mov	C,	Acc.7
	mov	B.3,	C
	clr	Acc.7
	mov	SBUF, 	A
	mov	R7, 	#01H		; Transmit status register
	pop	Acc
	pop	PSW
	reti	


;============================================================================
; Obsluha preruseni od serioveho rozhrani
;----------------------------------------------------------------------------

SIHan:	push	PSW
	push	Acc

Tra:	jnb	TI,	Rec
	clr	TI
Tra0:	cjne	R7,	#01H,	Tra1
	mov	P0, 	#11101111B	; Send bits  0 16 15 14 13 12 11 10
	nop
	mov	A,  	P2
	mov	P0, 	#11111111B
	mov	C,	Acc.7
	mov	B.2,	C
	clr	Acc.7
	mov	SBUF, 	A
	mov	R7,	#02H
	ljmp	Rec

Tra1:	cjne	R7,	#02H,	Tra2
	mov	P0, 	#11111011B	; Send bits  0 26 25 24 23 22 21 20
	nop
	mov	A,  	P2
	mov	P0, 	#11111111B
	mov	C,	Acc.7
	mov	B.1,	C
	clr	Acc.7
	mov	SBUF, 	A
	mov	R7,	#04H
	ljmp	Rec

Tra2:	cjne	R7,	#04H,	Tra3
	mov	P0, 	#11111110B	; Send bits  0 36 35 34 33 32 31 30
	nop
	mov	A,  	P2
	mov	P0, 	#11111111B
	mov	C,	Acc.7
	mov	B.0,	C
	clr	Acc.7
	mov	SBUF, 	A
	mov	R7,	#08H
	ljmp	Rec

Tra3:	cjne	R7,	#08H,	Rec
	mov	A,	B
	mov	SBUF,   A		; Send bits  1  0  0  0  37 27 17 07
	mov	R7,	#00H
	ljmp 	Rec
		
Rec:	jnb	RI,	Fin
	clr	RI
	mov	A,	SBUF
	jnb	Acc.7,	Rec0

	mov	C,	Acc.0
	mov	B0.7,	C
	mov	C,	Acc.1
	mov	B1.7,	C
	mov	C,	Acc.2
	mov	B2.7,	C
	mov	C,	Acc.3
	mov	B3.7,	C

	mov	P2,	B0
	mov	P0, 	#01111111B
	mov	P0,	#11111111B
	mov	P2,	B1
	mov	P0,	#11011111B
	mov	P0,	#11111111B
	mov	P2,	B2
	mov	P0,	#11110111B
	mov	P0,	#11111111B
	mov	P2,	B3
	mov	P0,	#11111101B
	mov	P0,	#11111111B
	mov	P2,	#0ffH

	mov	R6,	#01H
	ljmp	Fin

Rec0:	cjne	R6,	#01H,	Rec1
	mov	B0,	SBUF
	mov	R6,	#02H
	ljmp	Fin

Rec1:	cjne	R6,	#02H,	Rec2
	mov	B1,	SBUF
	mov	R6,	#04H
	ljmp	Fin

Rec2:	cjne	R6,	#04H,	Rec3
	mov	B2,	SBUF
	mov	R6,	#08H
	ljmp	Fin

Rec3	cjne	R6,	#08H,	Fin
	mov	B3,	SBUF
	mov	R6,	#00H	
	ljmp	Fin

Fin:	pop	Acc
	pop	PSW	
	reti	


;============================================================================
; Hlavni program
;----------------------------------------------------------------------------

Init:	
	mov	SP,	#08h
	mov	TMOD,	#21H		; Timer 1 ...  8-bit auto reloading
					; Timer 0 ... 16-bit 
	mov	TCON,	#50H		; Timers 0,1 Run
	mov	TH0,    #246		; T0 ... 400 Hz interrupt
	mov	TL0,	#255
	mov	A,	87h
	orl	A,	#80h
	mov	87h,	A		; set SMOD to double Baud rate
	mov	TH1,	#255		; 57600 Bd pro 11.0592 MHz
	mov	TL1,	#255		; 57600 Bd pro 11.0592 MHz
	mov 	SCON,	#050H		; 8-bit UART 
	mov	IE,	#92H		; T0 interrupt enable
					; Serial interface interrupt enable

	mov	B0,	#0
	mov	B1,	#0
	mov	B2,	#0
	mov	B3,	#0

	mov	R6,	#00H		; Receive  status register
	mov	R7,	#00H		; Transmit status register


Loop: 	sjmp $


